@@ -92,27 +92,58 @@ class GEOGRIDDATA(list):
 		return self.GEOGRID['properties']
 
 	def grid_size(self):
+		'''
+		Returns size of the grid (total numer of cells).
+		'''
 		return len(self.get_geogrid()['features'])
 
 	def get_type_info(self):
+		'''
+
+		'''
 		return self.get_geogrid_props()['types']
 
 	def get_type_set(self):
+		'''
+		Returns set with all types defined in GEOGRID.
+		'''
 		return set(self.get_geogrid_props()['types'])
 
 	def number_of_types(self):
 		return len(self.get_type_set())
 
-	def check_type_validity(self,quietly=True):
+	def check_type_validity(self,raise_error=True):
+		'''
+		Checks if all types in the given GEOGRIDDATA object correspond to a type defined in GEOGRID.
+		This function raises an error by default.
+
+		Parameters
+		----------
+		raise_error: boolean, defaults to `True`
+			If False, it will not raise the error by return a boolean of whether the types are valid or not. 
+		'''
 		non_defined_cells = set([cell['name'] for cell in self]).difference(self.get_type_set())
-		if len(non_defined_cells)==0:
-			return True
+		if len(non_defined_cells)!=0:
+			if raise_error:
+				raise NameError('Some types defined in GEOGRIDDATA do not match those in GEOGRID\n. Unrecognized types:',non_defined_cells)
+			else:
+				return False
 		else:
-			if not quietly:
-				print('Unrecognized types:',non_defined_cells)
-			return False
+			if not raise_error:
+				return True
+
 
 	def check_id_validity(self,quietly=True):
+		'''
+		Checks if all ids are in GEOGRIDDATA or if some are missing by comparing the number of unique ids of the current object with the grid size as return by :func:`brix.GEOGRIDDATA.grid_size`.
+		Does not raise an error, but returns a boolean. 
+		See :func:`brix.GEOGRIDDATA.fill_missing_cells`
+
+		Returns
+		-------
+		validity: boolean
+			If `False`, the number of unique ids does not match the grid size. 
+		'''
 		n_unique_ids = len(set([cell['id'] for cell in self]))
 		if n_unique_ids==self.grid_size():
 			return True
@@ -135,17 +166,20 @@ class GEOGRIDDATA(list):
 
 	def remap_colors(self):
 		'''
-		Forces the colors to match the define colors of the cell type. 
-		Requires that GEOGRIDDATA is set
+		Forces the colors to match the define colors of the cell type.
+		Requires that GEOGRIDDATA is set.
 		'''
 		if self.GEOGRID is None:
 			raise NameError('GEOGRIDDATA object does not have GEOGRID attribute.')
-		if not self.check_type_validity(quietly=False):
-			raise NameError('Type not found in table definition.')
+		self.check_type_validity()
 		GEOGRID = self.GEOGRID
 		for cell in self:
+			if 'color' in cell.keys():
+				current_color = cell['color']
 			h = GEOGRID['properties']['types'][cell['name']]['color'].replace('#','')
 			color = list(int(h[i:i+2], 16) for i in (0, 2, 4))
+			if len(current_color)==4:
+				color.append(current_color[-1]) #used to handle user-defined transparencies
 			cell['color'] = color
 
 	def remap_interactive(self):
@@ -256,6 +290,7 @@ class GEOGRIDDATA(list):
 		non_interative_id = sorted(non_interative_id)[::-1]
 		for i in non_interative_id:
 			del self[i]
+		self.df = None
 		return self
 
 	def bounds(self,bbox=False,buffer_percent=None):
@@ -485,15 +520,16 @@ class Handler(Thread):
 			indicatorName = ('0000'+str(len(self.indicators)+1))[-4:]
 
 		if I.tableHandler is not None:
-			warn(f'Indicator {indicatorName} has a table linked to it. Remember you do not need to link_table when using the Handler class')
+			warn(f'Indicator {indicatorName} has a table linked to it. This functionality will be deprecated soon.')
 
 		if indicatorName in self.indicators.keys():
 			warn(f'Indicator {indicatorName} already exists and will be overwritten')
 
+		I.table_name = self.table_name
 		self.indicators[indicatorName] = I
 		if test:
 			geogrid_data = self._get_grid_data()
-			if I.indicator_type not in set(['numeric','heatmap','access']):
+			if I.indicator_type not in set(['numeric','heatmap','access','hybrid']):
 				raise NameError('Indicator type should either be numeric, heatmap, or access. Current type: '+str(I.indicator_type))
 			try:
 				if I.is_composite:
@@ -541,7 +577,7 @@ class Handler(Thread):
 		else:
 			return I.return_indicator(geogrid_data)
 
-	def _format_geojson(self,new_value):
+	def _format_geojson(self,new_value,indicator_name):
 		'''
 		Formats the result of the return_indicator function into a valid geojson (not a cityIO geojson)
 		'''
@@ -629,41 +665,84 @@ class Handler(Thread):
 			}
 		'''
 		I = self.indicators[indicator_name]
+		new_value_raw = I.return_indicator(geogrid_data)
 
 		if I.indicator_type in ['access','heatmap']:
-			new_value = I.return_indicator(geogrid_data)
-			new_value = self._format_geojson(new_value)
-			return [new_value]
+			return self._new_value_heatmap(new_value_raw,I,indicator_name)
+
 		elif I.indicator_type in ['numeric']:
-			new_value = I.return_indicator(geogrid_data)
-			if isinstance(new_value,list)|isinstance(new_value,tuple):
-				for i in range(len(new_value)):
-					val = new_value[i]
-					if not isinstance(val,dict):
-						try:
-							json.dumps(val)
-							new_value[i] = {'value':val}
-						except:
-							warn('Indicator return invalid type:'+str(indicator_name))
-					if ('indicator_type' not in val.keys())&(I.indicator_type is not None):
-						val['indicator_type'] = I.indicator_type
-					if ('viz_type' not in val.keys())&(I.viz_type is not None):
-						val['viz_type'] = I.viz_type
-				return list(new_value)
+			return self._new_value_numeric(new_value_raw,I,indicator_name)
+
+		elif I.indicator_type in ['hybrid']:
+			if   'access'  in new_value_raw.keys():
+				new_value_heatmap = new_value_raw['access']
+			elif 'heatmap' in new_value_raw.keys():
+				new_value_heatmap = new_value_raw['heatmap']
 			else:
-				if not isinstance(new_value,dict):
+				raise NameError('No heatmap value found for hybrid indicator:',indicator_name)
+			new_value_heatmap = self._new_value_heatmap(new_value_heatmap,I,indicator_name)
+
+			if 'numeric' in new_value_raw.keys():
+				new_value_numeric = new_value_raw['numeric']
+			else:
+				raise NameError('No numeric value found for hybrid indicator:',indicator_name)
+			new_value_numeric = self._new_value_numeric(new_value_numeric,I,indicator_name)
+
+			return {'numeric':new_value_numeric,'heatmap':new_value_heatmap}
+
+	def _new_value_heatmap(self,new_value,I,indicator_name):
+		'''
+		Handles multiple formats of a new_value for a heatmap indicator.
+		GEOPANDAS.GEODATAFRAME IS ALSO SUPPORTED BUT HAS NOT BEEN TESTED
+
+		Parameters
+		----------
+		new_value: object
+			Object returned by some subclass of :func:`brix.Indicator.return_indicator` when :attr:`brix.Indicator.indicator_type` is `heatmap` or `access`.
+		'''
+		if isinstance(new_value, gpd.GeoDataFrame):
+			new_value = json.loads(new_value.to_json())
+		new_value = self._format_geojson(new_value,indicator_name)
+		return [new_value]
+
+
+	def _new_value_numeric(self,new_value,I,indicator_name):
+		'''
+		Handles multiple formats of a new_value for a numeric indicator.
+
+		Parameters
+		----------
+		new_value: object
+			Object returned by some subclass of :func:`brix.Indicator.return_indicator` when :attr:`brix.Indicator.indicator_type` is `numeric`.
+		'''
+		if isinstance(new_value,list)|isinstance(new_value,tuple):
+			for i in range(len(new_value)):
+				val = new_value[i]
+				if not isinstance(val,dict):
 					try:
-						json.dumps(new_value)
-						new_value = {'value':new_value}
+						json.dumps(val)
+						new_value[i] = {'value':val}
 					except:
 						warn('Indicator return invalid type:'+str(indicator_name))
-				if ('name' not in new_value.keys()):
-					new_value['name'] = indicator_name
-				if ('indicator_type' not in new_value.keys())&(I.indicator_type is not None):
-					new_value['indicator_type'] = I.indicator_type
-				if ('viz_type' not in new_value.keys())&(I.viz_type is not None):
-					new_value['viz_type'] = I.viz_type
-				return [new_value]
+				if ('indicator_type' not in val.keys())&(I.indicator_type is not None):
+					val['indicator_type'] = I.indicator_type
+				if ('viz_type' not in val.keys())&(I.viz_type is not None):
+					val['viz_type'] = I.viz_type
+			return list(new_value)
+		else:
+			if not isinstance(new_value,dict):
+				try:
+					json.dumps(new_value)
+					new_value = {'value':new_value}
+				except:
+					warn('Indicator return invalid type:'+str(indicator_name))
+			if ('name' not in new_value.keys()):
+				new_value['name'] = indicator_name
+			if ('indicator_type' not in new_value.keys())&(I.indicator_type is not None):
+				new_value['indicator_type'] = I.indicator_type
+			if ('viz_type' not in new_value.keys())&(I.viz_type is not None):
+				new_value['viz_type'] = I.viz_type
+			return [new_value]
 
 	def get_GEOGRID_EDGES(self):
 		'''
@@ -724,7 +803,7 @@ class Handler(Thread):
 
 	def get_indicator_values(self,geogrid_data=None,include_composite=False):
 		'''
-		Returns the current values of numeric indicators. Used for developing a composite indicator.
+		Returns the current values of NUMERIC indicators. Used for developing a composite indicator.
 
 		Parameters
 		----------
@@ -739,10 +818,14 @@ class Handler(Thread):
 		if geogrid_data is None:
 			geogrid_data = self._get_grid_data()
 		new_values_numeric = []
+
 		for indicator_name in self.indicators:
 			I = self.indicators[indicator_name]
 			if (I.indicator_type not in ['access','heatmap'])&(not I.is_composite):
-				new_values_numeric += self._new_value(geogrid_data,indicator_name)
+				if I.indicator_type=='hybrid':
+					new_values_numeric += self._new_value(geogrid_data,indicator_name)['numeric']
+				else:
+					new_values_numeric += self._new_value(geogrid_data,indicator_name)
 		indicator_values = {i['name']:i['value'] for i in new_values_numeric}
 		if include_composite:
 			for indicator_name in self.indicators:
@@ -776,7 +859,11 @@ class Handler(Thread):
 		for indicator_name in self.indicators:
 			try:
 				I = self.indicators[indicator_name]
-				if I.indicator_type in ['access','heatmap']:
+				if I.indicator_type in ['hybrid']:
+					new_value_hybrid = self._new_value(geogrid_data,indicator_name)
+					new_values_heatmap += new_value_hybrid['heatmap']
+					new_values_numeric += new_value_hybrid['numeric']
+				elif I.indicator_type in ['access','heatmap']:
 					new_values_heatmap += self._new_value(geogrid_data,indicator_name)
 				elif not I.is_composite:
 					new_values_numeric += self._new_value(geogrid_data,indicator_name)
@@ -805,7 +892,7 @@ class Handler(Thread):
 			if len(new_values_heatmap)!=0:
 				current_access = self.see_current(indicator_type='access')
 				self.previous_access = current_access
-				current_access = self._format_geojson(current_access)
+				current_access = self._format_geojson(current_access,None)
 				new_values_heatmap = [current_access]+new_values_heatmap
 
 		new_values_heatmap = self._combine_heatmap_values(new_values_heatmap)
@@ -1010,7 +1097,6 @@ class Handler(Thread):
 		new_values = self.update_package(append=append)
 
 		if len(new_values['numeric'])!=0:
-
 			r = requests.post(self.cityIO_post_url+'/indicators', data = json.dumps(new_values['numeric']))
 
 		if len(new_values['heatmap']['features'])!=0:
@@ -1123,19 +1209,25 @@ class Handler(Thread):
 		else:
 			self._listen(showFront=showFront)
 
-	def reset_geogrid_data(self):
+	def reset_geogrid_data(self,override_verification=True):
 		'''
 		Resets the GEOGRIDDATA endpoint to the initial value.
 		If the GEOGRIDDATA has not been updated, this will update it. 
+
+		Parameters
+		----------
+		override_verification: boolean, defaults to `True`
+			If True, it will ensure the object defined in GEOGRID/features is a valid GEOGRIDDATA object.
+			If False, it will post the object in GEOGRID/features to GEOGRIDDATa without any verification. 
 		'''
 		geogrid_data = []
 		for i,cell in enumerate(self.get_GEOGRID()['features']):
 			cell = cell['properties']
 			cell['id'] = i
 			geogrid_data.append(cell)
-		self.post_geogrid_data(geogrid_data)
+		self.post_geogrid_data(geogrid_data,override_verification=override_verification)
 
-	def post_geogrid_data(self,geogrid_data):
+	def post_geogrid_data(self,geogrid_data,override_verification=False):
 		'''
 		Posts the given geogrid_data object, ensuring that the object is valid.
 
@@ -1145,20 +1237,20 @@ class Handler(Thread):
 		----------
 		geogrid_data: dict
 			Dictionary corresponding to a valid :class:`brix.GEOGRIDDATA` object.
+		override_verification: boolean, defaults to `False`
+			If True, it will override the verification of the input as a valid object.
 		'''
-		geogrid_data = GEOGRIDDATA(geogrid_data)
-		geogrid_data.set_geogrid(self.get_GEOGRID())
-
-		if not geogrid_data.check_type_validity(quietly=False):
-			raise NameError('Type not found in table definition.')
-
-		if not geogrid_data.check_id_validity():
-			geogrid_data.fill_missing_cells()
+		if not override_verification:
+			geogrid_data = GEOGRIDDATA(geogrid_data)
+			geogrid_data.set_geogrid(self.get_GEOGRID())
+			geogrid_data.check_type_validity()
 			if not geogrid_data.check_id_validity():
-				raise NameError('IDs do not match.')
+				geogrid_data.fill_missing_cells()
+				if not geogrid_data.check_id_validity():
+					raise NameError('IDs do not match.')
 
-		geogrid_data.remap_colors()
-		geogrid_data.remap_interactive()
+			geogrid_data.remap_colors()
+			geogrid_data.remap_interactive()
 
 		geogrid_data = list(geogrid_data)
 
@@ -1241,7 +1333,8 @@ class Indicator:
 		pass
 
 	def return_indicator(self,geogrid_data):
-		'''User defined function. This function defines the value of the indicator as a function of the table state passed as `geogrid_data`. Function must return either a dictionary, a list, or a number. When returning a dict follow the format: ``{'name': 'Indicator_NAME', 'value': 1.00}``. 
+		'''
+		User defined function. This function defines the value of the indicator as a function of the table state passed as `geogrid_data`. Function must return either a dictionary, a list, or a number. When returning a dict follow the format: ``{'name': 'Indicator_NAME', 'value': 1.00}``. 
 
 		Parameters
 		----------
@@ -1253,10 +1346,33 @@ class Indicator:
 		indicator_value : list, dict, or float
 			Value of indicator or list of values. When returning a dict, please use the format ``{'name': 'Indicator Name', 'value': indicator_value}``. When returning a list, please return a list of dictionaries in the same format. 
 		'''
-		if self.return_indicator_user is not None:
-			return self.return_indicator_user(geogrid_data)
+		if self.indicator_type=='hybrid':
+			try:
+				out = {}
+				out['heatmap'] = self.return_indicator_heatmap(geogrid_data)
+				out['numeric'] = self.return_indicator_numeric(geogrid_data)
+			except:
+				out = {}
+				out['numeric'] = self.return_indicator_numeric(geogrid_data)
+				out['heatmap'] = self.return_indicator_heatmap(geogrid_data)
+			return out
 		else:
-			return {}
+			if self.return_indicator_user is not None:
+				return self.return_indicator_user(geogrid_data)
+			else:
+				return {}
+
+	def return_indicator_numeric(self,geogrid_data):
+		'''
+		Placeholder for user to define.
+		'''
+		return []
+
+	def return_indicator_heatmap(self,geogrid_data):
+		'''
+		Placeholder for user to define.
+		'''
+		return None
 
 	def set_return_indicator(self,return_indicator):
 		'''
@@ -1302,6 +1418,7 @@ class Indicator:
 		table_name: str or :class:`brix.Handler`
 			Name of the table or Handler object.
 		'''
+		warn('Indicator.link_table will be deprecated soon. Please use Handler class.')
 		if (table_name is None) & (self.table_name is None):
 			raise NameError('Please provide a table_name to link')
 		if table_name is None:
@@ -1313,6 +1430,7 @@ class Indicator:
 
 	def get_table_properties(self):
 		'''Gets table properties from the linked table. See :func:`brix.Indicator.link_table` and :func:`brix.Handler.get_table_properties`.'''
+		warn('Indicator.get_table_properties will be deprecated soon. Please use Handler.get_geogrid_props()[\'header\'].')
 		if (self.tableHandler is None)& (self.table_name is None):
 			raise NameError('No table linked: use Indicator.link_table(table_name)')
 		elif (self.tableHandler is None)& (self.table_name is not None):
@@ -1337,6 +1455,7 @@ class Indicator:
 		geogrid_data : str or pandas.DataFrame
 			Data that will be passed to the :func:`brix.Indicator.return_indicator` function by the :class:`brix.Handler` when deployed.
 		'''
+		warn('Indicator.get_geogrid_data will be deprecated soon. Please use Handler.get_geogrid_data.')
 		include_geometries     = self.requires_geometry if include_geometries is None else include_geometries
 		with_properties        = self.requires_geogrid_props if with_properties is None else with_properties
 		
